;;; ---"mDist" command by Stig B. Madsen---
;;; ---TDIST: Select objects and return total length of all lines and arcs contained in the selection.
;;; ---MDIST: Obtain distances by picking points and return summarized report
;;; ---LDIST: Summarize lengths of individually picked lines and arcs
;;; ---MDSETUP: Options for commands above.
;;; -----------------------------------------------------
;;; -----------------------------------------------------
(setq *md:box* T
      *md:color*
       7
      *md:delta*
       T
      *md:redraw*
       nil
      *md:dist*	0.0
)

(defun drawbox (pt aspect / x y npt alist)
  (if (or (null *md:color*) (/= (type *md:color*) 'INT))
    (setq *md:color* 7)
  )
  (setq x (car pt))
  (setq y (cadr pt))
  (setq npt (list (- x aspect) (+ y aspect)))
  (setq alist (cons npt alist))
  (setq npt (list (+ x aspect) (+ y aspect)))
  (setq alist (cons npt alist))
  (setq alist (cons npt alist))
  (setq npt (list (+ x aspect) (- y aspect)))
  (setq alist (cons npt alist))
  (setq alist (cons npt alist))
  (setq npt (list (- x aspect) (- y aspect)))
  (setq alist (cons npt alist))
  (setq alist (cons npt alist))
  (setq npt (list (- x aspect) (+ y aspect)))
  (setq alist (cons npt alist))
  (setq alist (cons *md:color* alist))
  (grvecs alist)
)

(defun drawline	(pt1 pt2 / llist)
  (if (null *md:color*)
    (setq *md:color* 7)
  )
  (setq llist nil)
  (setq llist (list (* *md:color* -1) pt1 pt2))
  (grvecs llist)
)

(defun arclen (entl / ept spt rad tmp)
  (setq	ept (cdr (assoc 51 entl))
	spt (cdr (assoc 50 entl))
	rad (cdr (assoc 40 entl))
	tmp (- ept spt)
  )
  (if (< tmp 0)
    (setq tmp (+ (* pi 2.0) tmp))
  )
  (* rad tmp)
)

(defun C:TDIST (/ sset a thedist tdist)
  (setq sset (ssget '((-4 . "<OR")(0 . "LINE") (0 . "ARC")(-4 . "OR>"))))
  (setq a 0
	tdist 0.0)
  (repeat (sslength sset)
    (setq ent  (ssname sset a)
	  entl (entget ent)
    )
    (cond
      ((= (cdr (assoc 0 entl)) "LINE")
       (setq pt1     (cdr (assoc 10 entl))
	     pt2     (cdr (assoc 11 entl))
	     thedist (distance pt1 pt2)
       )
      )
      ((= (cdr (assoc 0 entl)) "ARC")
       (setq thedist (arclen entl))
      )
    )
    (setq tdist (+ thedist tdist))
    (setq a (1+ a))
  )
  (mapcar 'princ (list "\nTotal distance of " a " objects: " (rtos tdist)))
  (setq *md:dist* tdist)
(princ)
  )

(defun C:MDIST
       (/ thedist tdist ldist dlist aspect pt1 pt2 fpt a scram cmd)
  (setq	thedist	0.0
	tdist 0.0
	ldist 0.0
	cmd (getvar "CMDECHO")
  )
  (setvar "CMDECHO" 0)
  (setq	aspect
	 (* (/ (getvar "VIEWSIZE") (cadr (getvar "SCREENSIZE"))) 4)
  )
  (setq pt2 T)
(while (/= (type pt1) 'LIST)
  (if (and *md:dist* (/= *md:dist* 0.000))
    (progn
      (initget "Continue")
      (setq pt1	(getpoint (strcat "\nFirst point or Continue ("
				  (rtos *md:dist*) "): "
			  )
		)
      )
      (cond ((= pt1 "Continue")
	     (setq tdist *md:dist*
		   *md:dist* nil
	     )
	     (princ (strcat "Continuing from " (rtos tdist)))
	    )
      )
    )
    (setq pt1 (getpoint "\nFirst point: "))
  )
)
  (if pt1
    (setq fpt pt1)
  )
  (if (and pt1 *md:box*)
    (drawbox pt1 aspect)
  )
  (setq a 2)
  (while (and pt1 pt2)
    (setq dlist (cons pt1 dlist))
    (initget "Close")
    (setq
      pt2 (getpoint (strcat "\nNext point or Close (" (itoa a) "): ")
		    pt1
	  )
    )
    (if	(= pt2 "Close")
      (setq pt2	fpt
	    scram T
      )
    )
    (if	pt2
      (progn
	(setq thedist (distance pt1 pt2))
	(setq tdist (+ thedist tdist))
	(if *md:box*
	  (drawbox pt2 aspect)
	)
	(drawline pt1 pt2)
	(setq dx (- (car pt1) (car pt2)))
	(setq dy (- (cadr pt1) (cadr pt2)))
	(if (/= a 2)
	  (setq	sumtxt
		 (strcat "Sum of last two distances: "
			 (rtos (+ thedist ldist))
		 )
	  )
	  (setq sumtxt "")
	)
	(prompt	(strcat	"\nDistance: "
			(rtos thedist)
			"  Angle from last point: "
			(angtos (* 180.0 (/ (angle pt1 pt2) pi)))
			"\n"
			sumtxt
			(if *md:delta*
			  (strcat "\nDelta X: "
				  (rtos dx)
				  "  Delta Y: "
				  (rtos dy)
			  )
			  ""
			)
			"  Total distance: "
			(rtos tdist)
		)
	)
	(setq ldist thedist)
	(setq pt1 pt2)
	(setq a (1+ a))
	(if scram
	  (setq	pt1 nil
		pt2 nil
	  )
	)
      )
    )
  )
  (setq *md:dist* tdist)
  (if *md:redraw*
    (command "REDRAW")
  )
  (setvar "CMDECHO" cmd)
  (princ)
)

(defun C:LDIST
       (/ dlist ent entl thedist tdist pt1 pt2 aspect dx dy elist)
  (setq	thedist	0.0
	tdist 0.0
	ldist 0.0
	cmd (getvar "CMDECHO")
  )
  (setvar "CMDECHO" 0)
  (setq	aspect
	 (* (/ (getvar "VIEWSIZE") (cadr (getvar "SCREENSIZE"))) 4)
  )
  (while (setq ent (car (entsel "\nSelect line or arc: ")))
    (if	ent
      (progn
	(setq elist (cons ent elist))
	(setq entl (entget ent))
	(cond ((= (cdr (assoc 0 entl)) "LINE")
	       (setq pt1     (cdr (assoc 10 entl))
		     pt2     (cdr (assoc 11 entl))
		     thedist (distance pt1 pt2)
	       )
	      )
	      ((= (cdr (assoc 0 entl)) "ARC")
	       (setq cpt     (cdr (assoc 10 entl))
		     rad     (cdr (assoc 40 entl))
		     pt1     (polar cpt (cdr (assoc 50 entl)) rad)
		     pt2     (polar cpt (cdr (assoc 51 entl)) rad)
		     thedist (arclen entl)
	       )
	       (redraw ent)
	      )
	      (T
	       (princ "*Invalid object*")
	       (setq ent nil)
	      )
	)
	(if ent
	  (progn
	    (redraw ent 3)
	    (setq tdist (+ thedist tdist))
	    (if	*md:box*
	      (drawbox pt1 aspect)
	    )
	    (if	*md:box*
	      (drawbox pt2 aspect)
	    )
	    (setq dx (- (car pt1) (car pt2)))
	    (setq dy (- (cadr pt1) (cadr pt2)))
	    (princ (strcat "\nLength:  "
			   (rtos thedist)
;;;			   "  Angle:  "
;;;			   (angtos (* 180.0 (/ (angle pt1 pt2) pi)))
			   (if *md:delta*
			     (strcat "\nDelta X: "
				     (rtos dx)
				     "  Delta Y: "
				     (rtos dy)
			     )
			     ""
			   )
			   "\nTotal:  "
			   (rtos tdist)
		   )
	    )
	    (setq ldist thedist)
	  )
	)
      )
    )
  )
  (if elist
    (foreach n elist (redraw n 4))
  )
  (if *md:redraw*
    (command "REDRAW")
  )
  (setvar "CMDECHO" cmd)
  (princ)
)

(defun C:MDSETUP	(/ ask1 ask col)
  (while (not ask1)
    (initget "eXit Box Color Redraw Delta ?")
    (setq ask1 (getkword "\nSetup [?/Box/Color/Redraw/Delta/<eXit>]: "))
    (cond
      ((or (= ask1 "Delta"))
       (initget "ON Off")
       (setq ask (getkword (strcat "\nToggle delta report "
				   (if *md:delta*
				     "ON/<Off>"
				     "<ON>/Off"
				   )
				   ": "
			   )
		 )
       )
       (cond ((null ask) (setq *md:delta* (not *md:delta*)))
	     ((= ask "ON") (setq *md:delta* T))
	     (T (setq *md:delta* nil))
       )
       (setq ask T)
      )
      ((= ask1 "Box")
       (initget "ON Off")
       (setq ask (getkword (strcat "\nDraw endpoint boxes "
				   (if *md:box*
				     "ON/<Off>"
				     "<ON>/Off"
				   )
				   ": "
			   )
		 )
       )
       (cond ((null ask) (setq *md:box* (not *md:box*)))
	     ((= ask "ON") (setq *md:box* T))
	     (T (setq *md:box* nil))
       )
      )
      ((= ask1 "Color")
       (if (or (null *md:color*) (/= (type *md:color*) 'INT))
	 (setq *md:color* 7)
       )
       (while (null col)
	 (setq
	   col (getint (strcat "\nSpecify ghost line color (0-256) <"
			       (itoa *md:color*)
			       ">: "
		       )
	       )
	 )
	 (cond
	   (col
	    (if	(<= 0 col 256)
	      (setq *md:color* col)
	      (progn (setq col nil) (princ "\nInvalid color number"))
	    )
	   )
	   ((null col)
	    (if	(<= 0 *md:color* 256)
	      (setq col T)
	    )
	   )
	 )

       )
      )
      ((= ask1 "Redraw")
       (initget "ON Off")
       (setq ask (getkword (strcat "\nRedraw automatically? "
				   (if *md:redraw*
				     "ON/<Off>"
				     "<ON>/Off"
				   )
				   ": "
			   )
		 )
       )
       (cond ((null ask) (setq *md:redraw* (not *md:redraw*)))
	     ((= ask "ON") (setq *md:redrawx* T))
	     (T (setq *md:redraw* nil))
       )

      )
      ((= ask1 "?")
       (princ (strcat "\nDraw box at endpoints: "
		      (if *md:box*
			"ON"
			"OFF"
		      )
	      )
       )
       (princ (strcat "\nGhost line color:      " (itoa *md:color*)))
       (princ (strcat "\nAutomatic redraw:      "
		      (if *md:redraw*
			"ON"
			"OFF"
		      )
	      )
       )
       (princ (strcat "\nShow delta report:     "
		      (if *md:delta*
			"ON"
			"OFF"
		      )
	      )
       )
       (setq ask1 nil)
      )
      ((or (null ask1) (= ask1 "eXit")) (setq ask1 T))
    )
  )
  (princ)
)

(princ (strcat "\nCommands:"
	       "\nMDSETUP: Options for commands below"
	       "\nTDIST:   Add lengths of selected lines and arcs"
	       "\nMDIST:   Add lengths from picked points"
	       "\nLDIST:   Add lengths of individually picked lines and arcs"
	       )
       )
(princ)
;;; -----------------------------------------------------
;;; -----------------------------------------------------
;;; ---end of "mDist" command by Stig B. Madsen---
;;; -----------------------------------------------------
;;; -----------------------------------------------------
